#pragma once

#include <cppargs/group/group.h>
#include <cppargs/options/member/member_option.h>

namespace cppargs
{

    /**
     * @brief Member group that is owned by other groups.
     */
    template<typename T_instance>
    struct member_owned_group
    {
    public:
        using instance_type = T_instance;
        using owner_type    = instance_owner<instance_type>;

        template<typename, typename, typename>
        friend struct member_group;

    protected:
        owner_type* _owner { nullptr };

    public:
        /**
         * @brief Destructor.
         */
        virtual ~member_owned_group() = default;
    };

    /**
     * @brief Member group that is owner of of other groups and options.
     */
    template<typename T_instance>
    struct member_owner_group
        : public instance_owner<T_instance>
    {
    public:
        using instance_type = T_instance;

        /**
         * @brief Get the instance of current parsed object.
         */
        virtual instance_type* instance() const override
            { return nullptr; }

        /**
         * @brief Destructor.
         */
        virtual ~member_owner_group() = default;
    };

    /**
     * @brief Member group that stores subgroups and options.
     */
    template<
        typename T_owner_instance,
        typename T_my_instance,
        typename T_resolve_instance_pred>
    struct member_group
        : public group
        , public member_owned_group<T_owner_instance>
        , public member_owner_group<T_my_instance>
    {
    public:
        using owner_instance_type           = T_owner_instance;
        using my_instance_type              = T_my_instance;
        using resolve_instance_pred_type    = T_resolve_instance_pred;
        using my_owned_group_type           = member_owned_group<owner_instance_type>;
        using sub_owned_group_type          = member_owned_group<my_instance_type>;
        using owner_group_type              = member_owner_group<my_instance_type>;
        using member_option_type            = member_option<my_instance_type>;

    private:
        resolve_instance_pred_type _predicate;

    public:
        /**
         * @brief Constructor.
         *
         * @param[in] p_meta    Meta data of the group.
         * @param[in] p_args    Sub groups and options.
         */
        template<typename... T_args>
        inline member_group(
            const group_meta& p_meta,
            resolve_instance_pred_type&& p_pred,
            T_args&&... p_args);

        /**
         * @brief Destructor.
         */
        virtual ~member_group() = default;

        /**
         * @brief Get the instance of current parsed object.
         */
        virtual my_instance_type* instance() const override;
    };

}
