#pragma once

#include <cppcore/conversion/string.h>

#include "option.h"

namespace cppargs
{

    /* argument_props */

    inline std::string enwrap_type(const std::string& type, bool optional)
    {
        auto opn = optional ? "(" : "<";
        auto cls = optional ? ")" : ">";
        return std::string(opn) + type + cls;
    }

    template<typename T_value, typename T_enable = void>
    struct argument_props
    {
        static std::string type(bool optional = false)
            { return "unknown"; };
    };

    template<>
    struct argument_props<std::string, void>
    {
        static std::string type(bool optional = false)
            { return enwrap_type("string", false); }
    };

    template<typename T_value>
    struct argument_props<T_value, std::enable_if_t<!std::is_class<T_value>::value>>
    {
        static std::string type(bool optional = false)
            { return enwrap_type(cppcore::type_helper<T_value>::name(), false); }
    };

    template<typename T_value>
    struct argument_props<std::list<T_value>, void>
    {
        static std::string type(bool optional = false)
        {
            return
                argument_props<T_value>::type(false) + ' ' +
                argument_props<T_value>::type(true) + ' ' +
                enwrap_type("...", true);
        }
    };

    template<typename T_value>
    struct argument_props<std::vector<T_value>, void>
    {
        static std::string type(bool optional = false)
        {
            return
                argument_props<T_value>::type(false) + ' ' +
                argument_props<T_value>::type(true) + ' ' +
                enwrap_type("...", true);
        }
    };

    /* prepare_impl */

    template<typename T_value, typename T_enable = void>
    struct prepare_impl
    {
        static inline option_meta prepare(const option_meta& meta, const T_value& value)
        {
            option_meta m = meta;
            if (m.arguments.empty())
                m.arguments = argument_props<T_value>::type(std::is_same<std::decay_t<T_value>, bool>::value);
            if (m.default_value.empty())
                m.set_default_value(value);
            return m;
        }
    };

    template<typename T_value, typename T_object>
    struct prepare_impl<
        T_value T_object::*,
        std::enable_if_t<std::is_default_constructible_v<T_object>>>
    {
        static inline option_meta prepare(const option_meta& meta, T_value T_object::*member)
        {
            option_meta m = meta;
            if (m.arguments.empty())
                m.arguments = argument_props<T_value>::type(std::is_same<T_value, bool>::value);
            if (m.default_value.empty())
                m.set_default_value<T_value>(T_object().*member);
            return m;
        }
    };

    /* option_meta */

    option_meta& option_meta::set_short_name(char value)
    {
        short_name = value;
        return *this;
    }

    option_meta& option_meta::set_long_name(const std::string& value)
    {
        long_name = value;
        return *this;
    }

    option_meta& option_meta::set_description(const std::string& value)
    {
        description = value;
        return *this;
    }

    option_meta& option_meta::set_arguments(const std::string& value)
    {
        arguments = value;
        return *this;
    }

    template <typename T_value>
    option_meta& option_meta::set_default_value(const T_value& value)
    {
        default_value = cppcore::to_string(value);
        return *this;
    }

    option_meta& option_meta::set_mandatory(bool value)
    {
        mandatory = value;
        return *this;
    }

    template<typename T_value>
    option_meta option_meta::prepare(const option_meta& meta, T_value&& value)
        { return prepare_impl<std::decay_t<T_value>>::prepare(meta, std::forward<T_value>(value)); }

    option::option(const option_meta& p_meta)
        : meta(p_meta)
        { }

}
