#pragma once

#include <string>
#include <cppargs/options/simple/simple_option.h>

namespace cppargs
{

    /**
     * @brief Specialized option using a predicate to parse the argument.
     */
    template<typename T_predicate>
    struct simple_predicate_option
        : public simple_option
    {
    public:
        using predicate_type = T_predicate;

    private:
        predicate_type _predicate;

    public:
        /**
         * @brief Constructor.
         *
         * @param[in] p_meta        Meta data of the option.
         * @param[in] args          Arguments to pass to the predicate constructor.
         */
        template<typename... T_args>
        inline simple_predicate_option(
            const option_meta& p_meta,
            T_args&&... args);

        /**
         * @brief Parse the current argument.
         *
         * @param[in,out] c     Context of the current argument.
         */
        virtual void parse(context& c) const override;
    };

}
